<?php

// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Get Contexts Web Service function
 *
 * @package    localgetcontexts
 * @copyright  2012 Francois Jacquet
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once($CFG->libdir . "/externallib.php");

class local_getcontexts_external extends external_api {

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function get_contexts_parameters() {
        return new external_function_parameters(
            array(
                'contexts' => new external_multiple_structure(
                    new external_single_structure(
                        array(
                            'contextlevel' => new external_value(PARAM_INT, 'the context level, for example CONTEXT_COURSE, or CONTEXT_MODULE.'),
                            'instance' => new external_value(PARAM_INT, 'the instance id. For contextlevel = CONTEXT_COURSE, this would be $course->id, for contextlevel = CONTEXT_MODULE, this would be $cm->id. And so on.'),
                        )
                    )
                )
            )
        );
    }


	/**
     * Get Contexts
     * @param array $contexts array of context description arrays (with contextlevel and instanceid)
     * @return array of contexts
     */
    public static function get_contexts($contexts) {
 
        $params = self::validate_parameters(self::get_contexts_parameters(), array('contexts'=>$contexts));
 
        $contexts = array();
 
		$contextlevels = array(CONTEXT_SYSTEM, CONTEXT_PERSONAL, CONTEXT_USER, CONTEXT_COURSECAT, CONTEXT_COURSE, CONTEXT_GROUP, CONTEXT_MODULE, CONTEXT_BLOCK);
		
        foreach ($params['contexts'] as $context) {
            $context = (object)$context;
			
            if (!is_int($context->contextlevel) || !in_array($context->contextlevel, $contextlevels)) {
                throw new invalid_parameter_exception('Invalid context level');
            }
            if (!is_int($context->instance) || $context->instance <= 0) {
                throw new invalid_parameter_exception('Invalid instance id');
            }
 
            $contextobject = get_context_instance($context->contextlevel, $context->instance, MUST_EXIST);
            self::validate_context($contextobject);
 
            // finally create the context
            $context->id = $contextobject->id;
            $context->path = $contextobject->path;
            $context->depth = $contextobject->depth;
            $contexts[] = (array)$context;
        }
 
        return $contexts;
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
	public static function get_contexts_returns() {
        return new external_multiple_structure(
            new external_single_structure(
                array(
                    'id' => new external_value(PARAM_INT, 'context id'),
					'contextlevel' => new external_value(PARAM_INT, 'the context level'),
					'instance' => new external_value(PARAM_INT, 'the instance id'),
					'path' => new external_value(PARAM_TEXT, 'path to context'),
					'depth' => new external_value(PARAM_INT, 'depth'),
                )
            )
        );
    }

}
